package jp.co.argo21.nautica.workflow.soap;

import javax.annotation.Resource;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import org.apache.log4j.Logger;

import jp.co.argo21.nautica.workflow.client.WorkflowServiceAccessor;
import jp.co.argo21.nautica.workflow.omg.WorkflowException;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.ProcDef;
import jp.co.argo21.nautica.workflow.soap.message.ProcessDefinitionState;
import jp.co.argo21.nautica.workflow.util.StringManager;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessDefinitionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

@WebService(serviceName = "DefinitionRepository",
        portName="DefinitionRepositoryPort",
        endpointInterface = "jp.co.argo21.nautica.workflow.soap.DefinitionRepository",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap",
        wsdlLocation = "WEB-INF/wsdl/DefinitionRepository.wsdl")
        
/**
 * Web サービスとして公開するインタフェースの SOAP バインディング実装クラスである。
 * XPDLファイルと名称のマッピングを管理しており、要求に応じて、XPDLをオブジェクト化して返す。
 */        
public class DefinitionRepositorySoapBindingImpl implements
		DefinitionRepository {
	
	/** ログ出力用オブジェクト */
	private static Logger log = Logger.getLogger(
			DefinitionRepositorySoapBindingImpl.class);

    @Resource
    private WebServiceContext context;
    
    /**
	 * プロセス定義状態を変更する。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param state プロセス定義状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@Override
	public void changeProcessDefinitionState(String session, String pdid,
			ProcessDefinitionState state) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidStateException, TransitionNotAllowedException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", ProcessDefinitionState=" + state + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.DefinitionRepository dr = accessor.getDefinitionRepository();
			dr.changeProcessDefinitionState(
					session, 
					pdid, 
					state.toWfmcProcessDefinitionState());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidStateException e) {
			String errMsg = StringManager.get("E0234");
			log.error(errMsg, e);
			throw e;
		} catch (TransitionNotAllowedException e) {
			String errMsg = StringManager.get("E0050");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * プロセス定義IDで特定された定義の現在の状態から遷移可能な
	 * プロセス定義状態一覧を返す。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @return プロセス定義状態の配列
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 */
	@Override
	public ProcessDefinitionState[] getProcessDefinitionStates(String session,
			String pdid) throws InvalidSessionException, InvalidProcessDefinitionException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid + ")";
		log.info(I4001);
		
		ProcessDefinitionState[] pdss = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.DefinitionRepository dr = accessor.getDefinitionRepository();
			pdss = SoapMessageConverter.convertToSoapMessage(
					dr.getProcessDefinitionStates(session, pdid));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002")
			+ "(ProcessDefinitionStates=" + pdss + ")";
   	log.info(I4002);
		return pdss;
		
	}

	/**
	 * 条件にマッチしたプロセス定義一覧を返す。
	 *
	 * @param session セッションID
	 * @param filter 検索用フィルター
	 * @return 検索されたプロセス定義概要の配列
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public ProcDef[] getProcessDefinitions(String session, Filter[] filter) throws InvalidSessionException, InvalidFilterException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		ProcDef[] pds = null;
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.DefinitionRepository dr = accessor.getDefinitionRepository();
			pds = SoapMessageConverter.convertToSoapMessage(
					dr.getProcessDefinitions(
							session, 
							SoapMessageConverter.convertToGroupFilter(filter)));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002")
   		+ "(ProcessDefinitions=" + pds + ")";
		log.info(I4002);
		return pds;
		
	}

	/**
	 * 指定された定義の登録処理を行い、エンジンに認識させる。
	 *
	 * @param session セッションID
	 * @param name ワークフロー定義名
	 * @param path ワークフロー定義の相対パス
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws WorkflowException プロセスインスタンスIDが有効でない場合
	 */
	@Override
	public void registerDefinition(String session, String name, String path) throws InvalidSessionException, WorkflowException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", WorkflowDefinitionName=" + name 
			+ ", WorkflowDefinitionPath=" + path +")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.DefinitionRepository dr = accessor.getDefinitionRepository();
			dr.registerDefinition(session, name, path);
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (WorkflowException e) {
			String errMsg = StringManager.get("E0189");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
		log.info(I4002);
		
	}
	
	public WebServiceContext getContext() {
		return context;
	}
	
}